/*
 * XORShift96Generator.cpp
 *
 *  Univ. Rennes, INSA Rennes, IETR VAADER
 *  Created on: Jun 8, 2014
 *  Author: Wassim Hamidouche
 *  Last Edited: Jun 21, 2019
 *  Author: Guillaume GAUTIER
 */

#include "XORShift96Generator.h"
#include <math.h>
#include <iostream>

XORShift96Generator::XORShift96Generator():x_key(123456789),y_key(362436069),z_key(521288629)
{
    Init();
}

void XORShift96Generator::Init() {
  x = x_key;
  y = y_key;
  z = z_key;
  current = 0;
  current_nb_bits = 0;
}

void XORShift96Generator::iterateGenerator(){
      unsigned long t;
      x ^= x << 16;
      x ^= x >> 5;
      x ^= x << 1;

      t = x;
      x = y;
      y = z;
      z = t ^ x ^ y;
      current = z;
      current_nb_bits = 32;
}

UInt  XORShift96Generator::get_key (UInt nb_bits) {
    unsigned int key = 0;
    if(nb_bits > 32) {
        std::cerr << "The Generator can not generate more than 32 bit("  << nb_bits << ")" << std::endl;
        return 0;
    }
    if(!nb_bits){
        return 0;
      }
    if(!current_nb_bits){
        iterateGenerator();
      }

    if(current_nb_bits >= nb_bits){
        key = current & (((UInt64)0x1<< nb_bits)-1);
        current >>= nb_bits;
        current_nb_bits -= nb_bits;
    } else {
        int e = current_nb_bits;
        key = current << (nb_bits-e);
        nb_bits -= current_nb_bits;
        iterateGenerator();
        key |= (current & (((UInt64)0x1<< (nb_bits-e))-1));
        current = current >> (nb_bits-e);
        current_nb_bits -= (nb_bits-e);
    }
    key &= (((UInt64)0x1<< nb_bits)-1);
    return key;
}
